/*
 * Copyright (C) 2023 Spacemit Limited
 * All Rights Reserved.
 */
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include "config.h"
#include "cam_log.h"
#include "board_option.h"

#include "cjson.h"
#define SAVE_FILE_PATH0 "/usr/share/camera_json/"
#define SAVE_FILE_PATH1 "/usr/share/"
#define SAVE_FILE_PATH2 "/tmp/"

int checkSpacemitBoard(void)
{
    FILE *stream;
    int ret = BOARD_DEFAULT, i;
    char *line = NULL;
    size_t len = 0;
    ssize_t read;

    stream = popen ("cat /sys/firmware/devicetree/base/model", "r");
    if ((read = getline (&line, &len, stream)) != -1) {
        for (i = 0; i < BOARD_MAX; i++) {
            if (strstr (line, "MUSE-Pi2")) {
                ret = BOARD_MUSE_PI2;
                printf ("the borad is MUSE-Pi2: %d, max board: %d\n", ret, BOARD_MAX);
                break;
            }
        }
    }
    pclose (stream);

    if (ret == BOARD_DEFAULT)
        printf ("the borad is default: %d, max board: %d\n", ret, BOARD_MAX);

    return ret;
}
static int getCppNodeConfig (struct testConfig *config, struct cJSON *root)
{
    struct cJSON *item = NULL, *child = NULL, *grandc = NULL;
    int num = 0, i, idx = 0, ret = 0;
    // char name[16] = "\0";
    char name[16];

    item = cJSON_GetObjectItem(root, "cpp_node");
    if (!item) {
        CLOG_ERROR("get cpp_node failed");
        ret = -1;
        goto out;
    }

    num = cJSON_GetArraySize(item);
    CLOG_INFO("cpp node num: %d", num);
    for (i = 0; i < num; i++) {
        child = cJSON_GetArrayItem(item, i);
        if (!child) {
            CLOG_ERROR("get cpp_node array %d failed", i);
            ret = -1;
            goto out;
        }
        grandc = cJSON_GetObjectItem(child, "name");
        if (!grandc) {
            CLOG_ERROR("get cpp%d name failed", i);
            ret = -1;
            goto out;
        }
        snprintf(name, sizeof(name), "%s", cjson_get_str(grandc));
        if (!strcmp(name, "cpp0")) {
            idx = 0;
        } else if (!strcmp(name, "cpp1"))
            idx = 1;
        else {
            CLOG_ERROR("invalid cpp name, only cpp0 or cpp1 valid");
            ret = -1;
            goto out;
        }

        grandc = cJSON_GetObjectItem(child, "enable");
        if (!grandc) {
            CLOG_ERROR("get cpp%d enable failed", i);
            ret = -1;
            goto out;
        }
        config->cppConfig[idx].enable = cjson_get_int(grandc);

        if (!config->cppConfig[idx].enable)
            continue;

        grandc = cJSON_GetObjectItem(child, "format");
        if (!grandc) {
            CLOG_ERROR("get cpp%d format failed", i);
            ret = -1;
            goto out;
        }
        snprintf(config->cppConfig[idx].format,
                sizeof(config->cppConfig[idx].format),
                "%s",
                cjson_get_str(grandc));

        grandc = cJSON_GetObjectItem(child, "src_from_file");
        if (!grandc) {
            CLOG_ERROR("get cpp%d src_from_file failed", i);
            ret = -1;
            goto out;
        }
        config->cppConfig[idx].srcFromFile = cjson_get_int(grandc);

        if (!config->cppConfig[idx].srcFromFile)
            continue;

        grandc = cJSON_GetObjectItem(child, "src_path");
        if (!grandc) {
            CLOG_WARNING("get cpp%d src_path failed", i);
            ret = -1;
            goto out;
        } else {
            snprintf(config->cppConfig[idx].srcFile,
                    sizeof(config->cppConfig[idx].srcFile),
                    "%s",
                    cjson_get_str(grandc));
        }

        grandc = cJSON_GetObjectItem(child, "size_width");
        if (!grandc) {
            CLOG_WARNING("get cpp%d size_width failed", i);
            ret = -1;
            goto out;
        } else {
            config->cppConfig[idx].width = cjson_get_int(grandc);
        }

        grandc = cJSON_GetObjectItem(child, "size_height");
        if (!grandc) {
            CLOG_WARNING("get cpp%d size_height failed", i);
            ret = -1;
            goto out;
        } else {
            config->cppConfig[idx].height = cjson_get_int(grandc);
        }
    }

out:
    return ret;
}

static int getIspNodeConfig (struct testConfig *config, struct cJSON *root)
{
    struct cJSON *item = NULL, *child = NULL, *grandc = NULL;
    int num = 0, i, idx = 0, ret = 0;
    // char name[16] = "\0";
    char name[16];

    item = cJSON_GetObjectItem(root, "isp_node");
    if (!item) {
        CLOG_ERROR("get isp_node failed");
        ret = -1;
        goto out;
    }
    num = cJSON_GetArraySize(item);
    CLOG_INFO("isp node num: %d", num);
    for (i = 0; i < num; i++) {
        child = cJSON_GetArrayItem(item, i);
        if (!child) {
            CLOG_ERROR("get isp_node array %d failed", i);
            ret = -1;
            goto out;
        }
        grandc = cJSON_GetObjectItem(child, "name");
        if (!grandc) {
            CLOG_ERROR("get isp%d name failed", i);
            ret = -1;
            goto out;
        }
        snprintf(name, sizeof(name), "%s", cjson_get_str(grandc));
        if (!strcmp(name, "isp0")) {
            idx = 0;
        } else if (!strcmp(name, "isp1"))
            idx = 1;
        else {
            CLOG_ERROR("invalid isp name, only isp0 or isp1 valid");
            ret = -1;
            goto out;
        }

        grandc = cJSON_GetObjectItem(child, "enable");
        if (!grandc) {
            CLOG_ERROR("get %s enable failed", name);
            ret = -1;
            goto out;
        }
        config->ispFeConfig[idx].enable = cjson_get_int(grandc);

        if (!config->ispFeConfig[idx].enable)
            continue;

        grandc = cJSON_GetObjectItem(child, "work_mode");
        if (!grandc) {
            CLOG_WARNING("get isp%d work_mode failed", idx);
            ret = -1;
            goto out;
        }
        if (!strcmp(cjson_get_str(grandc), "online")) {
            config->ispFeConfig[idx].workMode = ISP_WORKMODE_ONLINE;
        } else if (!strcmp(cjson_get_str(grandc), "rawdump")) {
            config->ispFeConfig[idx].workMode = ISP_WORKMODE_RAWDUMP;
        } else if (!strcmp(cjson_get_str(grandc), "offline_capture")) {
            config->ispFeConfig[idx].workMode = ISP_WORKMODE_OFFLINE_CAPTURE;
        } else if (!strcmp(cjson_get_str(grandc), "offline_preview")) {
            config->ispFeConfig[idx].workMode = ISP_WORKMODE_OFFLINE_PREVIEW;
        } else if (!strcmp(cjson_get_str(grandc), "ccic")) {
            config->ispFeConfig[idx].workMode = ISP_WORKMODE_CCIC;
        } else if (!strcmp(cjson_get_str(grandc), "slice_capture")) {
            config->ispFeConfig[idx].workMode = ISP_WORKMODE_SLICE_CAPTURE;
        } else {
            CLOG_ERROR("invalid isp work mode, valid modes: online, rawdump, offline_preview, offline_capture, ccic, slice_capture");
            ret -1;
            goto out;
        }

        if (config->ispFeConfig[idx].workMode == ISP_WORKMODE_ONLINE ||
            config->ispFeConfig[idx].workMode == ISP_WORKMODE_RAWDUMP ||
			config->ispFeConfig[idx].workMode == ISP_WORKMODE_CCIC) {
            // need sensor info
            grandc = cJSON_GetObjectItem(child, "sensor_name");
            if (!grandc) {
                CLOG_WARNING("get isp%d sensor_name failed", idx);
                ret = -1;
                goto out;
            }
            snprintf(config->ispFeConfig[idx].sensorName,
                    sizeof(config->ispFeConfig[idx].sensorName),
                    "%s",
                    cjson_get_str(grandc));

            grandc = cJSON_GetObjectItem(child, "sensor_id");
            if (!grandc) {
                CLOG_ERROR("get %s sensor_id failed", name);
                ret = -1;
                goto out;
            }
            config->ispFeConfig[idx].sensorId = cjson_get_int(grandc);

            grandc = cJSON_GetObjectItem(child, "sensor_work_mode");
            if (!grandc) {
                CLOG_ERROR("get %s sensor_work_mode failed", name);
                ret = -1;
                goto out;
            }
            config->ispFeConfig[idx].sensorWorkMode = cjson_get_int(grandc);

            grandc = cJSON_GetObjectItem(child, "fps");
            if (!grandc) {
                CLOG_WARNING("get %s fps failed", name);
            }
            config->ispFeConfig[idx].fps = cjson_get_int(grandc);
        }

        if (config->ispFeConfig[idx].workMode == ISP_WORKMODE_CCIC) {
            grandc = cJSON_GetObjectItem(child, "vc_mode");
            if (!grandc) {
                CLOG_ERROR("get %s vc_mode failed", name);
                ret = -1;
                goto out;
            }
            config->ispFeConfig[idx].vcMode = cjson_get_int(grandc);
		}

        if (config->ispFeConfig[idx].workMode == ISP_WORKMODE_OFFLINE_PREVIEW) {
            grandc = cJSON_GetObjectItem(child, "src_file");
            if (!grandc) {
                CLOG_WARNING("get isp%d src_file failed", idx);
                ret = -1;
                goto out;
            }
            snprintf(config->ispFeConfig[idx].srcFile,
                    sizeof(config->ispFeConfig[idx].srcFile),
                    "%s",
                    cjson_get_str(grandc));

            grandc = cJSON_GetObjectItem(child, "in_width");
            if (!grandc) {
                CLOG_ERROR("get isp%d in_width failed", idx);
                ret = -1;
                goto out;
            }
            config->ispFeConfig[idx].inWidth = cjson_get_int(grandc);

            grandc = cJSON_GetObjectItem(child, "in_height");
            if (!grandc) {
                CLOG_ERROR("get isp%d in_height failed", idx);
                ret = -1;
                goto out;
            }
            config->ispFeConfig[idx].inHeight = cjson_get_int(grandc);

            grandc = cJSON_GetObjectItem(child, "bit_depth");
            if (!grandc) {
                CLOG_ERROR("get isp%d bit_depth failed", idx);
                ret = -1;
                goto out;
            }
            config->ispFeConfig[idx].bitDepth = cjson_get_int(grandc);
        }

        grandc = cJSON_GetObjectItem(child, "format");
        if (!grandc) {
            CLOG_ERROR("get isp%d format failed", idx);
            ret = -1;
            goto out;
        }
        snprintf(config->ispFeConfig[idx].format,
                sizeof(config->ispFeConfig[idx].format),
                "%s",
                cjson_get_str(grandc));

        grandc = cJSON_GetObjectItem(child, "out_width");
        if (!grandc) {
            CLOG_ERROR("get isp%d out_width failed", idx);
            ret = -1;
            goto out;
        }
        config->ispFeConfig[idx].outWidth = cjson_get_int(grandc);

        grandc = cJSON_GetObjectItem(child, "out_height");
        if (!grandc) {
            CLOG_ERROR("get isp%d out_height failed", idx);
            ret = -1;
            goto out;
        }
        config->ispFeConfig[idx].outHeight = cjson_get_int(grandc);
    }

out:
    return ret;
}

static int getSensorNodeConfig (struct testConfig *config, struct cJSON *root)
{
    struct cJSON *item = NULL, *child = NULL, *grandc = NULL;
    int num = 0, i, idx = 0, ret = 0;
    char name[16] = "\0";

    item = cJSON_GetObjectItem(root, "sensor_node");
    if (!item) {
        CLOG_INFO("no sensor_node, use default config");
        config->snrConfig[0].flashEnable = 0;
        config->snrConfig[0].vcmEnable = 0;
        config->snrConfig[1].flashEnable = 0;
        config->snrConfig[1].vcmEnable = 0;
        config->snrConfig[0].snrI2cAddr = -1;
        config->snrConfig[1].snrI2cAddr = -1;
        config->useSnrNode = 0;
    } else {
        num = cJSON_GetArraySize(item);
        CLOG_INFO("sensor node num: %d", num);

        for (i = 0; i < num; i++) {
            child = cJSON_GetArrayItem(item, i);
            if (!child) {
                CLOG_ERROR("get sensor_node array %d failed", i);
                ret = -1;
                goto out;
            }
            // sensor config
            grandc = cJSON_GetObjectItem(child, "sensor_name");
            if (!grandc) {
                CLOG_ERROR("get sensor%d name failed", i);
                ret = -1;
                goto out;
            }
            snprintf(config->snrConfig[i].sensorName,
                    sizeof(config->snrConfig[i].sensorName),
                    "%s",
                    cjson_get_str(grandc));

            grandc = cJSON_GetObjectItem(child, "sensor_i2c_addr");
            if (!grandc) {
                config->snrConfig[i].snrI2cAddr = -1;
            } else {
                config->snrConfig[i].snrI2cAddr = cjson_get_int(grandc);
            }

            //vcm config
            grandc = cJSON_GetObjectItem(child, "vcm_name");
            if (!grandc) {
                config->snrConfig[i].vcmEnable = 0;
            } else {
                config->snrConfig[i].vcmEnable = 1;
                snprintf(config->snrConfig[i].vcmName,
                        sizeof(config->snrConfig[i].vcmName),
                        "%s",
                        cjson_get_str(grandc));

                grandc = cJSON_GetObjectItem(child, "vcm_i2c_bus");
                if (!grandc) {
                    config->snrConfig[i].vcmI2cBus = -1;
                } else {
                    config->snrConfig[i].vcmI2cBus = cjson_get_int(grandc);
                }
                grandc = cJSON_GetObjectItem(child, "vcm_i2c_addr");
                if (!grandc) {
                    config->snrConfig[i].vcmI2cAddr = -1;
                } else {
                    config->snrConfig[i].vcmI2cAddr = cjson_get_int(grandc);
                }
            }

            //flash config
            grandc = cJSON_GetObjectItem(child, "flash_name");
            if (!grandc) {
                config->snrConfig[i].flashEnable = 0;
            } else {
                config->snrConfig[i].flashEnable = 1;

                snprintf(config->snrConfig[i].flashName,
                        sizeof(config->snrConfig[i].flashName),
                        "%s",
                        cjson_get_str(grandc));
            }
        }
        config->useSnrNode++;
    }
    if (config->useSnrNode == 1) {
        config->snrConfig[1].flashEnable = 0;
        config->snrConfig[1].vcmEnable = 0;

        config->snrConfig[1].snrI2cAddr = -1;
    }

out:
    return ret;
}

int getTestConfig(struct testConfig *config, char *jsonfile)
{
    int ret = 0;
    int num = 0, i, idx = 0;
    struct cJSON *root = NULL, *item = NULL, *child = NULL, *grandc = NULL;
    char dftFile[64] = "/tmp/sdktest.json";

    if (!jsonfile) {
        CLOG_INFO("using default jsonfile %s", dftFile);
        jsonfile = dftFile;
    }
    if (access(jsonfile, F_OK | R_OK)) {
        CLOG_ERROR("can not access jsonfile %s", jsonfile);
        return -1;
    }

    FILE *file = NULL;
    file = fopen(jsonfile, "r");
    if (file == NULL) {
        CLOG_ERROR("can not open jsonfile %s", jsonfile);
        return -1;
    }

    struct stat statbuf;
    stat(jsonfile, &statbuf);
    int fileSize = statbuf.st_size;
    CLOG_INFO("json size:%ld", sizeof(char) * fileSize + 1);

    char *jsonStr = (char *)malloc(sizeof(char) * fileSize + 1);
    if (jsonStr == NULL) {
        CLOG_ERROR("can not malloc %d for jsonfile %s", fileSize, jsonfile);
        fclose(file);
        return -1;
    }
    memset(jsonStr, 0, fileSize + 1);

    int size = fread(jsonStr, sizeof(char), fileSize, file);
    if (size == 0) {
        CLOG_ERROR("can not read jsonfile %s", jsonfile);
        fclose(file);
        return -1;
    }
    // CLOG_INFO("read json output: %s", jsonStr);
    fclose(file);

    root = cJSON_Parse(jsonStr);
    if (!root) {
        CLOG_ERROR("Error before: %s", cJSON_GetErrorPtr());
        free(jsonStr);
        return -1;
    }
    free(jsonStr);

    item = cJSON_GetObjectItem(root, "tuning_server_enable");
    if (!item) {
        CLOG_ERROR("get tuning_server_enable failed");
        ret = -1;
        goto out;
    }
    config->tuningServerEnalbe = cjson_get_int(item);

    item = cJSON_GetObjectItem(root, "show_fps");
    if (!item) {
        CLOG_ERROR("get show_fps failed");
        ret = -1;
        goto out;
    }
    config->showFps = cjson_get_int(item);

    item = cJSON_GetObjectItem(root, "auto_run");
    if (!item) {
        CLOG_ERROR("get auto_run failed");
        ret = -1;
        goto out;
    }
    config->autoRun = cjson_get_int(item);

    item = cJSON_GetObjectItem(root, "test_frame");
    if (!item) {
        CLOG_DEBUG("no test_frame. set test_frame: 500, dump_one_frame:250 as default");
        config->testFrame = 500;
        config->dumpFrame = 250;
    } else {
        config->testFrame = cjson_get_int(item);
        config->testFrame = (config->testFrame < 6) ? 6 : config->testFrame;

        item = cJSON_GetObjectItem(root, "dump_one_frame");
        if (!item) {
            CLOG_DEBUG("get dump_one_frame failed, set to %d", config->testFrame / 2);
            config->dumpFrame = config->testFrame / 2;
        } else {
            config->dumpFrame = cjson_get_int(item);
            config->dumpFrame = (config->dumpFrame > config->testFrame - 1) ? config->testFrame - 1 : config->dumpFrame;
        }
    }

    item = cJSON_GetObjectItem(root, "use_v4l");
    if (!item) {
        config->useV4l = 0;
    } else {
        config->useV4l = cjson_get_int(item) ? 1 : 0;
    }

    item = cJSON_GetObjectItem(root, "auto_detect");
    if (!item) {
        config->autoDetect = 0;
    } else {
        config->autoDetect = cjson_get_int(item) ? 1 : 0;
    }

    item = cJSON_GetObjectItem(root, "gpu_render");
    if (!item) {
        config->gpuRender = 0;
    } else {
        config->gpuRender = cjson_get_int(item) ? 1 : 0;
    }

    if (config->gpuRender) {
        item = cJSON_GetObjectItem(root, "render_width");
        if (!item) {
            config->renderW = 1280;
        } else {
            config->renderW = cjson_get_int(item) > 1920 ? 1920 : cjson_get_int(item);
        }
        item = cJSON_GetObjectItem(root, "render_height");
        if (!item) {
            config->renderH = 720;
        } else {
            config->renderH = cjson_get_int(item) > 1080 ? 1080 : cjson_get_int(item);
        }
    }
    ret = getCppNodeConfig (config, root);
    if (ret) {
        CLOG_ERROR("cpp_node parse error");
        goto out;
    }
    ret = getIspNodeConfig (config, root);
    if (ret) {
        CLOG_ERROR("isp_node parse error");
        goto out;
    }
    ret = getSensorNodeConfig (config, root);
    if (ret) {
        CLOG_ERROR("sensor_node parse error");
        goto out;
    }

out:
    cJSON_Delete(root);
    root = NULL;

    return ret;
}

int update_json_file(struct testConfig *config, char *jsonfile, char *sensors_name, int width, int height)
{
    int ret = 0;
    int num = 0, i, idx = 0;
    struct cJSON *root = NULL, *item = NULL, *child = NULL, *grandc = NULL;
    char combined_path[256];
    char filename[64];
    char tstr[24];

    FILE *file = NULL;

    if (!jsonfile || access(jsonfile, F_OK | R_OK)) {
        CLOG_ERROR("can not access jsonfile %s", jsonfile);
        return -1;
    }

    file = fopen(jsonfile, "r");
    if (file == NULL) {
        CLOG_ERROR("can not open jsonfile %s", jsonfile);
        return -1;
    }

    struct stat statbuf;
    stat(jsonfile, &statbuf);
    int fileSize = statbuf.st_size;

    char *jsonStr = (char *)malloc(sizeof(char) * fileSize + 1);

    if (jsonStr == NULL) {
        CLOG_ERROR("can not malloc %d for jsonfile %s", fileSize, jsonfile);
        fclose(file);
        return -1;
    }

    memset(jsonStr, 0, sizeof(char) * fileSize + 1);

    int size = fread(jsonStr, sizeof(char), fileSize, file);
    if (size == 0) {
        CLOG_ERROR("can not read jsonfile %s", jsonfile);
        fclose(file);
        return -1;
    }
    fclose(file);

    root = cJSON_Parse(jsonStr);
    if (!root) {
        CLOG_ERROR("Error before: %s", cJSON_GetErrorPtr());
        free(jsonStr);
        return -1;
    }
    free(jsonStr);

    item = cJSON_GetObjectItem(root, "isp_node");
    if (!item) {
        CLOG_ERROR("get isp_node failed");
        ret = -1;
        goto out;
    }
    num = cJSON_GetArraySize(item);
    if (!num) {
        CLOG_ERROR("get isp node num: %d !", num);
        ret = -1;
        goto out;
    }

    child = cJSON_GetArrayItem(item, 0);
    if (!child) {
        CLOG_ERROR("get isp_node array 0 failed");
        ret = -1;
        goto out;
    }
    // grandc = cJSON_GetObjectItem(child, "sensor_name");
    // if (!grandc) {
    //     CLOG_WARNING("get isp%d sensor_name failed", idx);
    //     ret = -1;
    //     goto out;
    // }
    cJSON_ReplaceItemInObject(child, "sensor_name", cJSON_CreateString(sensors_name));
    // CLOG_WARNING("set isp sensor_name %s", sensors_name);

    // grandc = cJSON_GetObjectItem(child, "out_width");
    // if (!grandc) {
    //     CLOG_ERROR("get isp%d out_width failed", idx);
    //     ret = -1;
    //     goto out;
    // }
    cJSON_ReplaceItemInObject(child, "out_width", cJSON_CreateNumber(width));

    // grandc = cJSON_GetObjectItem(child, "out_height");
    // if (!grandc) {
    //     CLOG_ERROR("get isp%d out_height failed", idx);
    //     ret = -1;
    //     goto out;
    // }
    cJSON_ReplaceItemInObject(child, "out_height", cJSON_CreateNumber(height));

    cJSON_DeleteItemFromObject(root, "auto_detect");

    tstr[0] = config->ispFeConfig[0].sensorId + 1 + '0';
    tstr[1] = '\0';
    snprintf(filename, sizeof(filename), "%s%s%s", "csi", tstr, "_camera_auto.json");

    for (i = 0; i < 3; i++) {
        ret = 0;
        if (i == 0) {
            snprintf(combined_path, sizeof(combined_path), "%s%s", SAVE_FILE_PATH0, filename);
        } else if (i == 1) {
            snprintf(combined_path, sizeof(combined_path), "%s%s", SAVE_FILE_PATH1, filename);
        } else {
            snprintf(combined_path, sizeof(combined_path), "%s%s", SAVE_FILE_PATH2, filename);
        }
        file = fopen(combined_path, "w");
        if (file == NULL) {
            CLOG_ERROR("Open %s file fail", combined_path);
            ret = -1;
        } else
            break;
    }
    if (ret)
        goto out;

    char *cjValue = cJSON_Print(root);

    ret = fwrite(cjValue, sizeof(char), strlen(cjValue), file);
    if (ret == 0) {
        CLOG_ERROR("write %s file fail", combined_path);
    } else {
        CLOG_INFO("save json to %s success", combined_path);
    }
    fclose(file);
out:
    cJSON_Delete(root);
    root = NULL;

    return ret;
}